@@ -1,44 +1,43 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Basic GEOmetric Tool  (bgeot)                                */
-/* File    :  bgeot_convex.h :  convex with points                         */
-/*     									   */
-/* Date : December 20, 1999.                                               */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 1999-2002  Yves Renard.                                   */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
-
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 1999-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file bgeot_convex.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date December 20, 1999.
+   @brief Convex objects (structure + vertices)
+*/
 
 #ifndef BGEOT_CONVEX_H__
 #define BGEOT_CONVEX_H__
 
 #include <bgeot_convex_structure.h>
 
-namespace bgeot
-{
-  
+namespace bgeot {
+
+  /** @defgroup convexes Convexes */
+  /** @addtogroup convexes */
+  /*@{*/
+
+  /// generic definition of a convex ( bgeot::convex_structure + vertices coordinates )
   template<class PT, class PT_TAB = std::vector<PT> > class convex {
   public :
     
@@ -59,16 +58,19 @@ namespace bgeot
     
   public :
     
-    ref_convex_pt_ct point_of_face(short_type i) const { 
+    ref_convex_pt_ct points_of_face(short_type i) const { 
       return ref_convex_pt_ct(pts.begin(), cvs->ind_points_of_face(i).begin(),
 			      cvs->ind_points_of_face(i).end() );
     }
     
+    /// Return "direct" points. These are the subset of points than can be used to build a direct vector basis. (rarely used)
     ref_convex_pt_ct dir_points(void) const { 
       return ref_convex_pt_ct(pts.begin(), cvs->dir_points_.begin(),
 			      cvs->dir_points_.end() );
     }
-    
+    /** Direct points for a given face.
+	@param i the face number.
+    */
     dref_convex_pt_ct dir_points_of_face(short_type i) const {
       return dref_convex_pt_ct(pts.begin(),
 			       cvs->ind_dir_points_of_face(i).begin(),
@@ -80,19 +82,23 @@ namespace bgeot
     PT_TAB &points(void) { return pts; }
     short_type nb_points(void) const { return cvs->nb_points(); }
     
-    void translate(const typename PT::vector_type &v);
+    //void translate(const typename PT::vector_type &v);
     //template <class CONT> void base_of_orthogonal(CONT &tab);
     convex(void) { }
+    /** Build a convex object.
+	@param c the convex structure.
+	@param t the points array.
+    */
     convex(pconvex_structure c, const PT_TAB &t) : cvs(c), pts(t) {}
     convex(pconvex_structure c) : cvs(c) {}
   };
-
-  template<class PT, class PT_TAB>
+  /*@}*/
+  /*template<class PT, class PT_TAB>
     void convex<PT, PT_TAB>::translate(const typename PT::vector_type &v) {
     typename PT_TAB::iterator b = pts.begin(), e = pts.end();
     for ( ; b != e ; ++b) *b += v;
   }
-
+  */
   /*  
   template<class PT, class PT_TAB> template<class CONT>
     void convex<PT, PT_TAB>::base_of_orthogonal(CONT &tab)
@@ -165,14 +171,28 @@ namespace bgeot
 	     convex_product_structure(cv1.structure(), cv2.structure()), tab);
   }
 
+  struct special_convex_structure_key_ : virtual public dal::static_stored_object_key {
+    pconvex_structure p;
+    virtual bool compare(const static_stored_object_key &oo) const {
+      const special_convex_structure_key_ &o
+	= dynamic_cast<const special_convex_structure_key_ &>(oo);
+      if (p < o.p) return true;  return false;
+    }
+    special_convex_structure_key_(pconvex_structure pp) : p(pp) {}
+  };
+
   template<class PT, class PT_TAB1, class PT_TAB2>
     convex<PT> convex_direct_product(const convex<PT, PT_TAB1> &cv1,
-				     const convex<PT, PT_TAB2> &cv2)
-  {
+				     const convex<PT, PT_TAB2> &cv2) {
     if (cv1.nb_points() == 0 || cv2.nb_points() == 0)
       throw std::invalid_argument(
 		     "convex_direct_product : null convex product");
-
+    if (!dal::exists_stored_object(cv1.structure()))
+      dal::add_stored_object(new special_convex_structure_key_(cv1.structure()),
+			     cv1.structure(), dal::AUTODELETE_STATIC_OBJECT);
+    if (!dal::exists_stored_object(cv2.structure()))
+      dal::add_stored_object(new special_convex_structure_key_(cv2.structure()),
+			     cv2.structure(), dal::AUTODELETE_STATIC_OBJECT);
     convex<PT> r(convex_product_structure(cv1.structure(), cv2.structure()));
     r.points().resize(r.nb_points());
     std::fill(r.points().begin(), r.points().end(), PT(r.structure()->dim()));
@@ -207,28 +227,6 @@ namespace bgeot
     return r;
   }
 
-  /* structures de reference.                                             */
-
-//   template<class PT> convex<PT> simplex_of_reference(dim_type nc)
-//   {
-//     convex<PT> res(simplex_structure(nc));
-//     res.points().resize(nc+1);
-//     PT null(nc); null.fill(0.0);
-//     std::fill(res.points().begin(), res.points().end(), null);
-//     for (int i = 1; i <= nc; ++i) (res.points()[i])[i-1] = 1.0;
-//     return res;
-//   }
-
-//   template<class PT> 
-//     convex<PT> parallelepiped_of_reference(dim_type nc)
-//   { /* optimisable. */
-//     if (nc == 1) return simplex_of_reference<PT>(1);
-//     else
-//       return convex_direct_product<PT>(parallelepiped_of_reference<PT>(nc-1),
-// 				          simplex_of_reference<PT>(1));
-//   }
-
-
 }  /* end of namespace bgeot.                                             */
 
 
