@@ -1,56 +1,49 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Dynamic Array Library (dal)                                  */
-/* File    :  dal_ref.h : Structures which refere to containers.           */
-/*     									   */
-/*                                                                         */
-/* Date : August 26, 2000.                                                 */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2000-2002  Yves Renard.                                   */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2000-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
 
 #ifndef DAL_REF_H__
 #define DAL_REF_H__
 
-/* *********************************************************************** */
-/* WARNING : modifiying the container infirm the validity of references.   */
-/* *********************************************************************** */
+/** @file dal_ref.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date August 26, 2000.
+ *  @brief Provide some simple pseudo-containers.
+ *  
+ *  WARNING : modifiying the container infirm the validity of references.
+ */
 
 
 #include <iterator>
 #include <dal_basic.h>
 
-namespace dal
-{
+namespace dal {
 
   /* ********************************************************************* */
   /* Simple reference.                                                     */
   /* ********************************************************************* */
 
-  template<typename ITER> class tab_ref
-  {
+  template<typename ITER> class tab_ref {
+
     protected :
 
       ITER begin_, end_;
@@ -66,8 +59,8 @@ namespace dal
 	                                                       difference_type;
       typedef ITER                            iterator;
       typedef ITER                            const_iterator;
-      typedef dal::reverse_iter<const_iterator> const_reverse_iterator;
-      typedef dal::reverse_iter<iterator> reverse_iterator;
+      typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+      typedef std::reverse_iterator<iterator> reverse_iterator;
       typedef size_t size_type;
     
       bool empty(void) const { return begin_ == end_; }
@@ -175,8 +168,8 @@ namespace dal
 //       typedef size_t size_type; 
 //       typedef tab_ref_index_iterator_<ITER> iterator;
 //       typedef iterator                          const_iterator;
-//       typedef dal::reverse_iter<const_iterator> const_reverse_iterator;
-//       typedef dal::reverse_iter<iterator> reverse_iterator;
+//       typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+//       typedef std::reverse_iterator<iterator> reverse_iterator;
     
 //     protected :
 
@@ -219,10 +212,7 @@ namespace dal
 //   };
 
 
-  /* ********************************************************************* */
-  /* Reference with reference on index.                                    */
-  /* ********************************************************************* */
-
+  /// iterator over a dal::tab_ref_index_ref<ITER,ITER_INDEX>
   template<typename ITER, typename ITER_INDEX>
     struct tab_ref_index_ref_iterator_
     {
@@ -283,6 +273,10 @@ namespace dal
     return tab_ref_index_ref_iterator_<ITER,ITER_INDEX>(it, it_i);
   }
 
+  /** indexed array reference (given a container X, and a set of indexes I, 
+      this class provides a pseudo-container Y such that
+      @code Y[i] = X[I[i]] @endcode
+  */
   template<typename ITER, typename ITER_INDEX> class tab_ref_index_ref {
   public :
     
@@ -296,8 +290,8 @@ namespace dal
     typedef size_t                                size_type;
     typedef tab_ref_index_ref_iterator_<ITER, ITER_INDEX>   iterator;
     typedef iterator                              const_iterator;
-    typedef dal::reverse_iter<const_iterator>     const_reverse_iterator;
-    typedef dal::reverse_iter<iterator>           reverse_iterator;
+    typedef std::reverse_iterator<const_iterator>     const_reverse_iterator;
+    typedef std::reverse_iterator<iterator>           reverse_iterator;
     
   protected :
 
@@ -395,6 +389,9 @@ namespace dal
     return tab_ref_reg_spaced_iterator_<ITER>(it, stride);
   }
 
+  /**
+     provide a "strided" view a of container
+  */
   template<typename ITER> class tab_ref_reg_spaced
   {
     public :
@@ -409,8 +406,8 @@ namespace dal
       typedef size_t size_type;
       typedef tab_ref_reg_spaced_iterator_<ITER> iterator;
       typedef iterator                          const_iterator;
-      typedef dal::reverse_iter<const_iterator> const_reverse_iterator;
-      typedef dal::reverse_iter<iterator> reverse_iterator;
+      typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+      typedef std::reverse_iterator<iterator> reverse_iterator;
     
     protected :
 
@@ -450,10 +447,7 @@ namespace dal
 
   };
 
-  /* ********************************************************************* */
-  /* Reference elements selected with a condition.                         */
-  /* ********************************************************************* */
-
+  /// iterator over a tab_ref_with_selection
   template<typename ITER, typename COND> 
     struct tab_ref_with_selection_iterator_ : public ITER
   {
@@ -478,41 +472,45 @@ namespace dal
     
   };
 
-  template<typename ITER, typename COND> class tab_ref_with_selection
-  {
-    public :
-
-      typedef typename std::iterator_traits<ITER>::value_type value_type;
-      typedef typename std::iterator_traits<ITER>::pointer    pointer;
-      typedef typename std::iterator_traits<ITER>::pointer    const_pointer;
-      typedef typename std::iterator_traits<ITER>::reference  reference;
-      typedef typename std::iterator_traits<ITER>::reference  const_reference;
-      typedef size_t  size_type;
-      typedef tab_ref_with_selection_iterator_<ITER, COND> iterator;
-      typedef iterator   const_iterator;
+  /**
+     given a container X and a predicate P, provide pseudo-container Y
+     of all elements of X such that P(X[i]).
+  */
+  template<typename ITER, typename COND> class tab_ref_with_selection {
+    
+  protected :
+    
+    ITER begin_, end_;
+    COND cond;
+    
+    
+    
+  public :
+    
+    typedef typename std::iterator_traits<ITER>::value_type value_type;
+    typedef typename std::iterator_traits<ITER>::pointer    pointer;
+    typedef typename std::iterator_traits<ITER>::pointer    const_pointer;
+    typedef typename std::iterator_traits<ITER>::reference  reference;
+    typedef typename std::iterator_traits<ITER>::reference  const_reference;
+    typedef size_t  size_type;
+    typedef tab_ref_with_selection_iterator_<ITER, COND> iterator;
+    typedef iterator   const_iterator;
+    
+    iterator begin(void) const
+    { iterator it(begin_, cond); it.forward(); return it; }
+    iterator end(void) const { return iterator(end_, cond); }
+    bool empty(void) const { return begin_ == end_; }
+    
+    value_type front(void) const { return *begin(); }
+    void pop_front(void) { ++begin_; begin_ = begin(); }
+    
+    COND &condition(void) { return cond; }
+    const COND &condition(void) const { return cond; }
+    
+    tab_ref_with_selection(void) {}
+    tab_ref_with_selection(const ITER &b, const ITER &e, const COND &c)
+      : begin_(b), end_(e), cond(c) { begin_ = begin(); }
     
-    protected :
-
-      ITER begin_, end_;
-      COND cond;
-
-    public :
-
-      iterator begin(void) const
-      { iterator it(begin_, cond); it.forward(); return it; }
-      iterator end(void) const { return iterator(end_, cond); }
-      bool empty(void) const { return begin_ == end_; }
-
-      value_type front(void) const { return *begin(); }
-      void pop_front(void) { ++begin_; begin_ = begin(); }
-
-      COND &condition(void) { return cond; }
-      const COND &condition(void) const { return cond; }
-   
-      tab_ref_with_selection(void) {}
-      tab_ref_with_selection(const ITER &b, const ITER &e, const COND &c)
-	: begin_(b), end_(e), cond(c) { begin_ = begin(); }
-
   };
 
 }
