@@ -1,35 +1,30 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  GEneric Tool for Finite Element Methods (Getfem)             */
-/* File    :  getfem_interpolation.h : interpolation beetween different    */
-/*            meshes or at least mesh-fems.                                */
-/*     									   */
-/* Date : October 15, 2001.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*          Julien Pommier, pommier@gmm.insa-tlse.fr                       */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2001-2004  Yves Renard, Julien Pommier.                   */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2001-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file getfem_interpolation.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>, Julien Pommier <Julien.Pommier@insa-toulouse.fr>
+   @date October 15, 2001.
+   @brief Interpolation of fields from a mesh_fem onto another.
+*/
 
 #ifndef GETFEM_INTERPOLATION_H__
 #define GETFEM_INTERPOLATION_H__
@@ -51,7 +46,7 @@ namespace getfem {
 
   protected :
     typedef gmm::abstract_null_type void_type;
-    const getfem_mesh &mesh;
+    const mesh &msh;
     std::vector<std::map<size_type, void_type> > pts_cvx;
     typedef std::map<size_type, void_type>::const_iterator map_iterator;
     std::vector<base_node> ref_coords;
@@ -71,7 +66,7 @@ namespace getfem {
      *        into account, else test them on the frontiere convexes.
      */
     void distribute(bool extrapolation = false);
-    mesh_trans_inv(const getfem_mesh &m):bgeot::geotrans_inv(1E-12),mesh(m) {}
+    mesh_trans_inv(const mesh &m) : bgeot::geotrans_inv(1E-12), msh(m) {}
   private :
     void add_point_with_id(base_node, size_type) {}
   };
@@ -79,14 +74,121 @@ namespace getfem {
 
   /* ********************************************************************* */
   /*								   	   */
-  /*	II. Interpolation.                                  		   */
+  /*	II. Interpolation of functions.                     		   */
+  /*									   */
+  /* ********************************************************************* */
+
+
+  template <typename VECT, typename F, typename M>
+  inline void interpolation_function__(const mesh_fem &mf, VECT &V,
+				       F &f, const dal::bit_vector &dofs,
+				       const M &, gmm::abstract_null_type) {
+    size_type Q = mf.get_qdim();
+    if (gmm::vect_size(V) != mf.nb_dof() || Q != 1)
+      DAL_THROW(failure_error, "Dof vector has not the right size");
+    for (dal::bv_visitor i(dofs); !i.finished(); ++i)
+      V[i] = f(mf.point_of_dof(i));
+  }
+
+  template <typename VECT, typename F, typename M>
+  inline void interpolation_function__(const mesh_fem &mf, VECT &V,
+				       F &f, const dal::bit_vector &dofs,
+				       const M &v, gmm::abstract_vector) {
+    size_type N = gmm::vect_size(v),  Q = mf.get_qdim();
+    if (gmm::vect_size(V) != mf.nb_dof()*N/Q)
+      DAL_THROW(failure_error, "Dof vector has not the right size");
+    for (dal::bv_visitor i(dofs); !i.finished(); ++i)
+      if (i % Q == 0)
+	gmm::copy(f(mf.point_of_dof(i)),
+		  gmm::sub_vector(V, gmm::sub_interval(i*N/Q, N)));
+  }
+
+  template <typename VECT, typename F, typename M>
+  inline void interpolation_function__(const mesh_fem &mf, VECT &V,
+				       F &f, const dal::bit_vector &dofs,
+				       const M &mm, gmm::abstract_matrix) {
+    size_type Nr = gmm::mat_nrows(mm), Nc = gmm::mat_ncols(mm), N = Nr*Nc;
+    size_type Q = mf.get_qdim();
+    base_matrix m(Nr, Nc);
+    if (gmm::vect_size(V) != mf.nb_dof()*N/Q)
+      DAL_THROW(failure_error, "Dof vector has not the right size");
+    for (dal::bv_visitor i(dofs); !i.finished(); ++i)
+      if (i % Q == 0) {
+	gmm::copy(f(mf.point_of_dof(i)), m);
+	for (size_type j = 0; j < Nc; ++j)
+	  gmm::copy(gmm::mat_col(m, j),
+		    gmm::sub_vector(V, gmm::sub_interval(i*N/Q+j*Nr, Nr)));
+      }
+  }
+
+  template <typename VECT, typename F, typename M>
+  inline void interpolation_function_(const mesh_fem &mf, VECT &V,
+				      F &f, const dal::bit_vector &dofs,
+				      const M &m) {
+    interpolation_function__(mf, V, f, dofs, m,
+			     typename gmm::linalg_traits<M>::linalg_type());
+  }
+
+#if GETFEM_PARA_LEVEL > 0
+  template <typename T>
+  void take_one_op(void *a, void *b, int *len, MPI_Datatype *) {
+    T aa = *((T*)a);
+    return aa ? aa : *((T*)b);
+  }
+
+  template <typename T>
+  inline MPI_Op mpi_take_one_op(T) {
+    static bool isinit = false;
+    static MPI_Op op;
+    if (!isinit) {
+      MPI_Op_create(take_one_op<T>, true, &op);
+      isinit = true;
+    }
+    return op;
+  }
+      
+  template <typename VECT> inline void MPI_MERGE_VECTOR(VECT& V) {
+    typedef typename gmm::linalg_traits<VECT>::value_type T;
+    std::vector<T> W(gmm::vect_size(V)); gmm::copy(V, W);
+    MPI_Allreduce(&(V[0]), &(W[0]), gmm::vect_size(V), mpi_type(T()),
+		  mpi_take_one_op<T>, MPI_COMM_WORLD);
+  }
+#else
+  template <typename VECT> inline void MPI_MERGE_VECTOR(VECT& ) {}
+#endif
+
+  // TODO : verify that rhs is a lagrange fem
+  /**
+     @brief interpolation of a function f on mf_target.
+     - mf_target must be of lagrange type.
+     - mf_target's qdim should be equal to the size of the return value of f,
+       or equal to 1
+     - V should have the right size
+     CAUTION: with the parallized version (GETFEM_PARA_LEVEL >= 2) the
+     resulting vector V is distributed.
+  */
+  template <typename VECT, typename F>
+  void interpolation_function(mesh_fem &mf_target, const VECT &V, F &f,
+			 mesh_region rg=mesh_region::all_convexes()) {
+    gmm::clear(const_cast<VECT &>(V));
+    mf_target.linked_mesh().intersect_with_mpi_region(rg);
+    dal::bit_vector dofs = mf_target.dof_on_set(rg);
+    if (dofs.card() > 0)
+      interpolation_function_(mf_target, const_cast<VECT &>(V), f, dofs,
+			      f(mf_target.point_of_dof(dofs.first())));
+    //MPI_MERGE_VECTOR(V);
+  }
+
+  /* ********************************************************************* */
+  /*								   	   */
+  /*	III. Interpolation between two meshes.                		   */
   /*									   */
   /* ********************************************************************* */
 
   /* ------------------------------ Interface -----------------------------*/
 
   /**
-     interpolation/extrapolation of (mf_source, U) on mf_target.
+     @brief interpolation/extrapolation of (mf_source, U) on mf_target.
      - mf_target must be of lagrange type.
      - mf_target's qdim should be equal to mf_source qdim, or equal to 1
      - U.size() >= mf_source.get_qdim()
@@ -101,7 +203,8 @@ namespace getfem {
 		     const VECTU &U, VECTV &V, bool extrapolation = false);
 
   /**
-     Build the interpolation matrix of mf_source on mf_target: the matrix M is
+     @brief Build the interpolation matrix of mf_source on mf_target.
+     the matrix M is
      such that (V = M*U) == interpolation(mf_source, mf_target, U, V).
 
      Useful for repeated interpolations.
@@ -151,42 +254,46 @@ namespace getfem {
     for (dal::bv_visitor cv(mf_source.convex_index()); !cv.finished(); ++cv) {
       bgeot::pgeometric_trans pgt=mf_source.linked_mesh().trans_of_convex(cv);
       pfem pf_s = mf_source.fem_of_element(cv);
+      if (!mf_target.convex_index().is_in(cv)) 
+	continue;
       pfem pf_t = mf_target.fem_of_element(cv);
       size_type nbd_s = pf_s->nb_dof(cv);
       size_type nbd_t = pf_t->nb_dof(cv);
-      ref_mesh_dof_ind_ct::iterator itdof;
+      mesh_fem::ind_dof_ct::const_iterator itdof;
+      size_type cvnbdof = mf_source.nb_dof_of_element(cv);
 
       if (version == 0) {
         coeff.resize(qqdim);
         for (size_type qq=0; qq < qqdim; ++qq) {
           coeff[qq].resize(nbd_s*qdim);
           itdof = mf_source.ind_dof_of_element(cv).begin();
-          for (size_type k = 0; k < mf_source.nb_dof_of_element(cv);
-               ++k, ++itdof) coeff[qq][k] = U[(*itdof)*qqdim+qq];
+          for (size_type k = 0; k < cvnbdof; ++k, ++itdof) {
+	    coeff[qq][k] = U[(*itdof)*qqdim+qq];
+	  }
         }
       }
       if (pf_s->need_G()) 
 	bgeot::vectors_to_base_matrix(G,
 			      mf_source.linked_mesh().points_of_convex(cv));
 
-      if (pf_s->target_dim() != 1 || pf_t->target_dim() != 1)
-	DAL_THROW(to_be_done_error,
-		  "vector FEM interpolation still to be done ... ");
+      if (pf_t->target_dim() != 1)
+	DAL_THROW(to_be_done_error, "won't interpolate on a vector FEM... ");
       pfem_precomp pfp = fppool(pf_s, pf_t->node_tab(cv));
-      fem_interpolation_context ctx(pgt,pfp,size_type(-1),G,cv);
+      fem_interpolation_context ctx(pgt,pfp,size_type(-1), G, cv);
       itdof = mf_target.ind_dof_of_element(cv).begin();
-      dof_source.assign(mf_source.ind_dof_of_element(cv).begin(), 
-                        mf_source.ind_dof_of_element(cv).end());
+      const mesh_fem::ind_dof_ct &idct
+	= mf_source.ind_dof_of_element(cv);
+      dof_source.assign(idct.begin(), idct.end());
       for (size_type i = 0; i < nbd_t; ++i, itdof+=mf_target.get_qdim()) {
 	size_type dof_t = *itdof*qmult;
         if (dof_t_done.is_in(*itdof)) continue;
         dof_t_done.add(*itdof);
-	/* faux dans le cas des lments vectoriel.                        */
 	ctx.set_ii(i);
 	if (version == 0) {
           for (size_type qq=0; qq < qqdim; ++qq) {
             pf_s->interpolation(ctx, coeff[qq], val, qdim);
-            for (size_type k=0; k < qdim; ++k) V[(dof_t + k)*qqdim+qq] = val[k];
+            for (size_type k=0; k < qdim; ++k)
+	      V[(dof_t + k)*qqdim+qq] = val[k];
           }
 	}
 	else {
@@ -196,12 +303,6 @@ namespace getfem {
             for (size_type j=0; j < dof_source.size(); ++j) {
               M(dof_t + k, dof_source[j]) = Mloc(k, j);
             }
-              /* does not work with col matrices..
-                 gmm::clear(gmm::mat_row(M, dof_t + k));
-                 gmm::copy(gmm::mat_row(Mloc, k),
-                 gmm::sub_vector(gmm::mat_row(M, dof_t+k),
-                 gmm::sub_index(mf_source.ind_dof_of_element(cv))));
-              */
 	  }
 	}
       }
@@ -222,7 +323,7 @@ namespace getfem {
 		     int version, bool extrapolation = false) {
 
     typedef typename gmm::linalg_traits<VECTU>::value_type T;
-    const getfem_mesh &mesh(mf_source.linked_mesh());
+    const mesh &msh(mf_source.linked_mesh());
     size_type qdim_s = mf_source.get_qdim();
     size_type qqdim = gmm::vect_size(U)/mf_source.nb_dof();
     
@@ -239,14 +340,15 @@ namespace getfem {
     base_tensor Z;
     std::vector<size_type> dof_source;
 
-    for (dal::bv_visitor cv(mesh.convex_index()); !cv.finished(); ++cv) {
-      bgeot::pgeometric_trans pgt=mesh.trans_of_convex(cv);
+    for (dal::bv_visitor cv(mf_source.convex_index()); !cv.finished(); ++cv) {
+      bgeot::pgeometric_trans pgt=msh.trans_of_convex(cv);
       mti.points_on_convex(cv, itab);
       if (itab.size() == 0) continue;
 
       pfem pf_s = mf_source.fem_of_element(cv);
+      //cerr << "pf_s = "<< pf_s << ", mf_source.cvidx = " << mf_source.convex_index().is_in(cv) << " cv=" << cv << "\n";
       if (pf_s->need_G()) 
-	bgeot::vectors_to_base_matrix(G, mesh.points_of_convex(cv));
+	bgeot::vectors_to_base_matrix(G, msh.points_of_convex(cv));
 
       fem_interpolation_context ctx(pgt, pf_s, base_node(), G, cv);
       if (version == 0) {
@@ -257,8 +359,9 @@ namespace getfem {
                     gmm::sub_index(mf_source.ind_dof_of_element(cv))), coeff[qq]);
         }
       }
-      dof_source.assign(mf_source.ind_dof_of_element(cv).begin(), 
-                        mf_source.ind_dof_of_element(cv).end());
+      const mesh_fem::ind_dof_ct &idct
+	= mf_source.ind_dof_of_element(cv);
+      dof_source.assign(idct.begin(), idct.end());
       for (size_type i = 0; i < itab.size(); ++i) {
 	size_type dof_t = itab[i];
 	if (dof_done.is_in(dof_t)) {
@@ -305,7 +408,7 @@ namespace getfem {
   void interpolation(const mesh_fem &mf_source, mesh_trans_inv &mti,
 		     const VECTU &U, VECTV &V, bool extrapolation = false) {
     base_matrix M;
-    if ((gmm::vect_size(U) % mf_source.nb_dof()) != 0 || gmm::vect_size(V) == 0)
+    if ((gmm::vect_size(U) % mf_source.nb_dof()) != 0 || gmm::vect_size(V)==0)
       DAL_THROW(dimension_error, "Dimensions mismatch");
     interpolation(mf_source, mti, U, V, M, 0, extrapolation);
   }
@@ -323,8 +426,8 @@ namespace getfem {
 		       const VECTU &U, VECTV &V, MAT &M,
 		       int version, bool extrapolation = false) {
 
-    const getfem_mesh &mesh(mf_source.linked_mesh());
-    getfem::mesh_trans_inv mti(mesh);
+    const mesh &msh(mf_source.linked_mesh());
+    getfem::mesh_trans_inv mti(msh);
     size_type qdim_s = mf_source.get_qdim(), qdim_t = mf_target.get_qdim();
     if (qdim_s != qdim_t && qdim_t != 1)
       DAL_THROW(failure_error, "Attempt to interpolate a field of dimension "
