@@ -1,34 +1,24 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  Generic Matrix Methods  (gmm)                                */
-/* File    :  gmm_solver_cg.h : conjugated gradient.                       */
-/*            Modified version of I.T.L. conjugated gradient.              */
-/*     									   */
-/* Date : October 13, 2002.                                                */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2002  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GMM++                                            */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2002-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 //=======================================================================
 // Copyright (C) 1997-2001
 // Authors: Andrew Lumsdaine <lums@osl.iu.edu> 
@@ -52,11 +42,16 @@
 // OR OTHER RIGHTS.
 //=======================================================================
 
+/**@file gmm_solver_cg.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date October 13, 2002.
+   @brief Conjugate gradient iterative solver. 
+*/
 #ifndef GMM_SOLVER_CG_H__
 #define GMM_SOLVER_CG_H__
 
 #include <gmm_kernel.h>
-#include <gmm_precond_diagonal.h>
+#include <gmm_iter.h>
 
 namespace gmm {
 
@@ -86,6 +81,10 @@ namespace gmm {
       rho = vect_hp(PS, z, r);
       copy(z, p);
 
+// #ifdef GMM_USES_MPI
+//       double t_ref, t_prec = MPI_Wtime(), t_tot = 0;
+//       static double tmult_tot = 0.0;
+// #endif
       while (!iter.finished_vect(r)) {
 
 	if (!iter.first()) { 
@@ -93,12 +92,76 @@ namespace gmm {
 	  rho = vect_hp(PS, z, r);
 	  add(z, scaled(p, rho / rho_1), p);
 	}
-	
+// #ifdef GMM_USES_MPI
+// t_ref = MPI_Wtime();
+//     cout << "mult CG " << endl;
+// #endif	
 	mult(A, p, q);
+// #ifdef GMM_USES_MPI
+//     tmult_tot += MPI_Wtime()-t_ref;
+//     cout << "tmult_tot CG = " << tmult_tot << endl;
+// #endif
 	a = rho / vect_hp(PS, q, p);	
 	add(scaled(p, a), x);
 	add(scaled(q, -a), r);
 	rho_1 = rho;
+
+// #ifdef GMM_USES_MPI
+// 	t_tot = MPI_Wtime() - t_prec;
+// 	cout << "temps CG : " << t_tot << endl; 
+// #endif
+	++iter;
+      }
+    }
+  }
+
+  template <typename Matrix, typename Matps, typename Precond, 
+            typename Vector1, typename Vector2>
+  void cg(const Matrix& A, Vector1& x, const Vector2& b, const Matps& PS,
+	  const gmm::identity_matrix &, iteration &iter) {
+
+    typedef typename temporary_dense_vector<Vector1>::vector_type temp_vector;
+    typedef typename linalg_traits<Vector1>::value_type T;
+
+    T rho, rho_1(0), a;
+    temp_vector p(vect_size(x)), q(vect_size(x)), r(vect_size(x));
+    iter.set_rhsnorm(gmm::sqrt(gmm::abs(vect_hp(PS, b, b))));
+
+    if (iter.get_rhsnorm() == 0.0)
+      clear(x);
+    else {
+      mult(A, scaled(x, T(-1)), b, r);
+      rho = vect_hp(PS, r, r);
+      copy(r, p);
+
+// #ifdef GMM_USES_MPI
+//       double t_ref, t_prec = MPI_Wtime(), t_tot = 0;
+//       static double tmult_tot = 0.0;
+// #endif
+      while (!iter.finished_vect(r)) {
+
+	if (!iter.first()) { 
+	  rho = vect_hp(PS, r, r);
+	  add(r, scaled(p, rho / rho_1), p);
+	}
+// #ifdef GMM_USES_MPI
+// t_ref = MPI_Wtime();
+//     cout << "mult CG " << endl;
+// #endif	
+	mult(A, p, q);
+// #ifdef GMM_USES_MPI
+//     tmult_tot += MPI_Wtime()-t_ref;
+//     cout << "tmult_tot CG = " << tmult_tot << endl;
+// #endif
+	a = rho / vect_hp(PS, q, p);	
+	add(scaled(p, a), x);
+	add(scaled(q, -a), r);
+	rho_1 = rho;
+
+// #ifdef GMM_USES_MPI
+// 	t_tot = MPI_Wtime() - t_prec;
+// 	cout << "temps CG : " << t_tot << endl; 
+// #endif
 	++iter;
       }
     }
