@@ -1,35 +1,30 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
-/* *********************************************************************** */
-/*                                                                         */
-/* Library :  LINK MeSsaGE library (linkmsg)                               */
-/* File    :  linkmsg.h : structure for managing Messages between objects. */
-/*     									   */
-/*                                                                         */
-/* Date : March 21, 1999.                                                  */
-/* Author : Yves Renard, Yves.Renard@gmm.insa-tlse.fr                      */
-/*                                                                         */
-/* *********************************************************************** */
-/*                                                                         */
-/* Copyright (C) 2001  Yves Renard.                                        */
-/*                                                                         */
-/* This file is a part of GETFEM++                                         */
-/*                                                                         */
-/* This program is free software; you can redistribute it and/or modify    */
-/* it under the terms of the GNU Lesser General Public License as          */
-/* published by the Free Software Foundation; version 2.1 of the License.  */
-/*                                                                         */
-/* This program is distributed in the hope that it will be useful,         */
-/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
-/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
-/* GNU Lesser General Public License for more details.                     */
-/*                                                                         */
-/* You should have received a copy of the GNU Lesser General Public        */
-/* License along with this program; if not, write to the Free Software     */
-/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,  */
-/* USA.                                                                    */
-/*                                                                         */
-/* *********************************************************************** */
+// -*- c++ -*- (enables emacs c++ mode)
+//========================================================================
+//
+// Copyright (C) 2001-2006 Yves Renard
+//
+// This file is a part of GETFEM++
+//
+// Getfem++ is free software; you can redistribute it and/or modify
+// it under the terms of the GNU Lesser General Public License as
+// published by the Free Software Foundation; version 2.1 of the License.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU Lesser General Public License for more details.
+// You should have received a copy of the GNU Lesser General Public
+// License along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301,
+// USA.
+//
+//========================================================================
 
+/**@file linkmsg.h
+   @author  Yves Renard <Yves.Renard@insa-toulouse.fr>
+   @date March 21, 1999.
+   @brief structure for managing Messages between objects.
+*/
 
 #ifndef LINKMSG_H__
 #define LINKMSG_H__
@@ -122,7 +117,7 @@ namespace lmsg
     typename RECEIVERTAB::const_tas_iterator it = receivers.tas_begin(),
       ite = receivers.tas_end();
     for (; it != ite; ++it)
-      if ((masks[it.index()])[int(msg)]) (*it)->receipt(msg);
+      if ((masks[it.index()])[T::ID]) (*it)->receipt(msg);
   }
 
   template<class RECEIVER>
