@@ -21,14 +21,14 @@ EXTENSION = 0.0;
 MESH_TYPE = 'GT_PK(3,1)';         % linear triangles
 NX = 3;            	          % space step.
 NZ = 4;
-MESH_NOISE = 0; % Set to one if you want to "shake" the mesh
+MESH_NOISED = 0; % Set to one if you want to "shake" the mesh
 FEM_TYPE = 'FEM_PK(3,2)';  % P1 for triangles
 FEM_TYPE_P = 'FEM_PK(3,1)';  % P1 for triangles
 DATA_FEM_TYPE = 'FEM_PK(3,1)'
 INTEGRATION = 'IM_TETRAHEDRON(6)'
-RESIDU = 1E-6;     	% residu for iterative solvers.
+RESIDUAL = 1E-6;     	% residu for iterative solvers.
 MAXITER = 15;
-USE_MULTIPLIERS=0
+DIRICHLET_VERSION=2
 NBSTEP = 1;
 ROOTFILENAME = 'nonlinear_elastostatic';     % Root of data files.
 VTK_EXPORT = 1 % export solution to a .vtk file ?
